#define GAIM_PLUGINS
#include "gaim.h"
#include <gtk/gtk.h>

#ifndef FESTIVAL_VOICES_PATH
#define FESTIVAL_VOICES_PATH "/usr/share/festival/voices"
#endif

#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <dirent.h>
#include <errno.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define debug(a) g_printerr("%s = %p\n",#a,a)
#define assert(a) g_assert(a)

enum IDS_MESSAGES {
	IDS_PLUGIN_NAME,
	IDS_PLUGIN_ABOUT,
	IDS_DURATION,
	IDS_VOICE_SELECTION,
	IDS_AVAILABLES_VOICES,
	IDS_ERROR_OPENING_VOICES_DIR,
	IDS_ERROR_OPENING_VOICE_DIR,
	IDS_NO_FESTIVAL,
	IDS_CLOSE,
	IDS_LAST
};

static FILE *festival_pf= NULL;
static GString *prefered_voice= NULL;
static float stretch_duration= 1.0;
static GtkWidget *window= NULL;

char *gaim_plugin_init(GModule *handle);
void gaim_plugin_unload(GModule *handle);
void gaim_plugin_config();
void received_im(struct gaim_connection *gc, char **who, char **what, void *m);
	
void set_stretch_duration(float duration);
void set_prefered_voice(const char *voice);

static void load_conf();
static void save_conf();

static void on_spinbutton_changed(GtkWidget *widget);
static void on_radio_clicked(GtkWidget *widget,GString *data);
static void on_radio_destroy(GtkWidget *widget,GString *data);
static gint on_window_destroy();
static void on_close_button_clicked(GtkWidget *widget);

static char* __(enum IDS_MESSAGES id);

char *es_messages[]= {
	"Parlante de mensajes"
	#ifdef VERSION
	" version " VERSION
	#endif
	,
	"Este aadido parla los mensajes que recibes.\n"
		"Para ms comodidad, una vez cargado,\n"
		"se recomienda deshabilitar los sonidos de notificacin.",
	"Duracin",
	"Seleccin de voz",
	"Voces disponibles:",
	"Error al abrir directorio de voces: %s",
	"Error al abrir directorio de voz: %s",
	"No pudo encontrar el ejecutable de festival",
	"Cerrar",
	""
};

char *en_messages[]= {
	"Message speaker"
	#ifdef VERSION
	" version " VERSION
	#endif
	,
	"This plugin speak your incoming messages.\n"
		"For your commodity, when is active,\n"
		"is recommended to disable notify sounds.",
	"Duration",
	"Voice selection",
	"Availables voices:",
	"Error opening voices directory: %s",
	"Error opening voice directory: %s",
	"Cannot find festival executable in the PATH",
	"Close",
	""
};

char *__(enum IDS_MESSAGES ids) {
	if(ids < 0 || ids >= IDS_LAST)
		return "";
	else
	if(strncmp("es",getenv("LANG"),2) == 0)
		return es_messages[ids];
	else
		return en_messages[ids];
}

char *name() {
	return __(IDS_PLUGIN_NAME);
}

char *description() {
	return __(IDS_PLUGIN_ABOUT);
}

void set_stretch_duration(float duration) {
	stretch_duration= duration;
	if(festival_pf) {
		fprintf(festival_pf,"(Parameter.set 'Duration_Stretch %2.1f)\n",stretch_duration);
		fflush(festival_pf);
	}
}

void set_prefered_voice(const char *voice) {
	if(prefered_voice)
		g_string_assign(prefered_voice, voice);
	else
		prefered_voice= g_string_new(voice);
	if(festival_pf) {
			fprintf(festival_pf,"(voice_%s)\n",prefered_voice->str);
			fflush(festival_pf);
	}
}

static void on_spinbutton_changed(GtkWidget *widget) {
	set_stretch_duration( gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(widget)) );
	save_conf();
}

static void on_radio_clicked(GtkWidget *widget,GString *voice) {
	if( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) ) {
		set_prefered_voice(voice->str);
		save_conf();
	}
}

static void on_radio_destroy(GtkWidget *widget,GString *data) {
	g_string_free(data,TRUE);
}

static gint on_window_destroy() {
	if(window)
		gtk_widget_destroy(window);
	window= NULL;
	return FALSE;
}

static void on_close_button_clicked(GtkWidget *widget) {
	on_window_destroy();
}

static void load_conf() {
	FILE *conf_file;
	GString *filename;
	char param[32],value[32];
	int code;

	filename= g_string_new("");
	g_string_sprintf(filename, "%s/.gaim/festival", getenv("HOME"));
	conf_file= fopen(filename->str,"r");
	g_string_free(filename,TRUE);
	if(!conf_file)
		return;
	while(TRUE) {
		code= fscanf(conf_file,"%31s",param);
		if(code == EOF)
			break;
		if( !strcmp("stretch_duration",param) ) {
			code= fscanf(conf_file,"%f",&stretch_duration);
			if(code == EOF)
				stretch_duration= 1.0;
			set_stretch_duration(stretch_duration);
				
		}
		else
		if( !strcmp("prefered_voice",param) ) {
			code= fscanf(conf_file,"%31s",&value);
			if(code != EOF)
				set_prefered_voice(value);
		}
	}
	fclose(conf_file);
}

static void save_conf() {
	FILE *conf_file;
	GString *filename;
	
	filename= g_string_new("");
	g_string_sprintf(filename, "%s/.gaim/festival", getenv("HOME"));
	conf_file= fopen(filename->str,"w");
	g_string_free(filename,TRUE);
	fprintf(conf_file,"prefered_voice %s\n",prefered_voice->str);
	fprintf(conf_file,"stretch_duration %2.1f\n",stretch_duration);
	fclose(conf_file);
	load_conf();
}

void received_im(struct gaim_connection *gc, char **who, char **what, void *m) {
	int i;
	GString *buffer= g_string_new("");
	g_string_sprintf(buffer,"(SayText \"%s\")\n",strip_html(*what));
	for(i=0;i<buffer->len;i++)
		if( strchr("",buffer->str[i]) )
			buffer->str[i] = ' ';
	if(festival_pf) {
		fprintf(festival_pf,buffer->str);
		fflush(festival_pf);
	}
	g_string_free(buffer,TRUE);
}

char *gaim_plugin_init(GModule *handle) {
	FILE *which_pf;
	char line[1024];
	which_pf= popen("which festival 2>/dev/null","r");
	fscanf(which_pf,"%1023s",line);
	pclose(which_pf);
	
	if( *line != '/')
		return __(IDS_NO_FESTIVAL);

	errno=0;
	festival_pf= popen("festival","w");
	if(errno) {
		pclose(festival_pf);
		return strerror(errno);
	}
	load_conf();
	gaim_signal_connect(handle, event_im_recv, (gpointer)received_im, NULL);
	return NULL;
}

void gaim_plugin_remove(GModule *handle) {
	if(festival_pf) {
		fprintf(festival_pf,"(quit)");
		pclose(festival_pf);
		festival_pf= 0;
	}
	if(prefered_voice) {
		g_string_free(prefered_voice,TRUE);
		prefered_voice= 0;
	}
}

void gaim_plugin_config() {
	GString
		*base_dir_name= NULL,
		*lang_dir_name= NULL;
	GtkWidget
		*scrolled_window,
		*config_vbox, *voices_vbox, *close_hbox, *stretch_duration_hbox,
		*stretch_duration_adjustment,
		*frame,
		*radio_button,
		*message_label,
		*stretch_duration_spinbutton,
		*close_button;
	GSList *radio_group=NULL;
	
	DIR *dir, *lang_dir;
	struct dirent *next_lang, *next_voice;
	int count_lang, count_voices;
	int i;
	int can_activate_voice= FALSE;
	
	if(window)
		return;
	GAIM_DIALOG(window);
	gtk_window_set_title(GTK_WINDOW(window), __(IDS_VOICE_SELECTION));
	gtk_window_set_policy(GTK_WINDOW(window),FALSE,TRUE,TRUE);
	
	config_vbox= gtk_vbox_new(FALSE,5);
	gtk_container_set_border_width(GTK_CONTAINER(config_vbox), 5);
	gtk_container_add(GTK_CONTAINER(window),config_vbox);

	base_dir_name= g_string_new(FESTIVAL_VOICES_PATH);
	lang_dir_name= g_string_new("");
	
	message_label= gtk_label_new( __(IDS_AVAILABLES_VOICES) );
	gtk_box_pack_start(GTK_BOX(config_vbox),message_label,FALSE,TRUE,3);
	
	close_hbox= gtk_hbox_new(FALSE,3);
	gtk_box_pack_end(GTK_BOX(config_vbox),close_hbox,FALSE,TRUE,3);
	
	close_button= gtk_button_new_with_label( __(IDS_CLOSE) );
	gtk_signal_connect(GTK_OBJECT(close_button),"clicked",GTK_SIGNAL_FUNC(on_close_button_clicked),NULL);
	gtk_box_pack_start(GTK_BOX(close_hbox),gtk_invisible_new(),TRUE,TRUE,3);
	gtk_box_pack_end(GTK_BOX(close_hbox),close_button,TRUE,TRUE,3);
	
	stretch_duration_hbox= gtk_hbox_new(FALSE,3);
	stretch_duration_adjustment= gtk_adjustment_new(1.0,0.3,10.0,0.1,1.0,1.0);
	stretch_duration_spinbutton= gtk_spin_button_new(GTK_ADJUSTMENT(stretch_duration_adjustment),0.1,1);
	gtk_signal_connect(GTK_OBJECT(stretch_duration_spinbutton),"changed",on_spinbutton_changed,NULL);
	gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(stretch_duration_spinbutton),GTK_UPDATE_IF_VALID);
	gtk_entry_set_editable(GTK_ENTRY(stretch_duration_spinbutton),FALSE);
	gtk_box_pack_start(GTK_BOX(stretch_duration_hbox),gtk_label_new(__(IDS_DURATION)),FALSE,FALSE,3);
	gtk_box_pack_start(GTK_BOX(stretch_duration_hbox),stretch_duration_spinbutton,FALSE,FALSE,3);
	gtk_box_pack_end(GTK_BOX(config_vbox),stretch_duration_hbox,FALSE,TRUE,3);
	
	dir= opendir(base_dir_name->str);
	if(!dir) {
		GString *error_name;
		error_name= g_string_new("");
		g_string_sprintf(error_name, __(IDS_ERROR_OPENING_VOICES_DIR) ,base_dir_name->str);
		message_label= gtk_label_new(error_name->str);
		g_string_free(error_name,TRUE);
		gtk_box_pack_start(GTK_BOX(config_vbox),message_label,FALSE,TRUE,3);
	}
	else {
		count_lang=0;
		while( ( next_lang = readdir(dir) ) !=0  ) {
			if(count_lang++ < 2)
				continue;
			frame= gtk_frame_new(next_lang->d_name);
			gtk_box_pack_start(GTK_BOX(config_vbox),frame,FALSE,TRUE,3);
			voices_vbox= gtk_vbox_new(FALSE,5);
			gtk_container_set_border_width(GTK_CONTAINER(voices_vbox), 5);
			gtk_container_add(GTK_CONTAINER(frame),voices_vbox);

			g_string_sprintf(lang_dir_name,"%s/%s",base_dir_name->str,next_lang->d_name);
		
			lang_dir= opendir(lang_dir_name->str);
			if(!lang_dir) {
				GString *error_name;
				error_name= g_string_new("");
				g_string_sprintf(error_name, __(IDS_ERROR_OPENING_VOICE_DIR) ,lang_dir_name->str);
				message_label= gtk_label_new(error_name->str);
				g_string_free(error_name,TRUE);
				gtk_box_pack_start(GTK_BOX(voices_vbox),message_label,FALSE,TRUE,3);
				break;
			}
			
			count_voices=0;
			while( ( next_voice = readdir(lang_dir) )!=0  ) {
				GString *voice_name;
				if(count_voices++ < 2)
					continue;
				
				radio_button= gtk_radio_button_new_with_label(radio_group,next_voice->d_name);
				
				if( prefered_voice && strcmp(prefered_voice->str,next_voice->d_name ) == 0) {
					gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(radio_button),TRUE );
					can_activate_voice= TRUE;
				}
				
				voice_name= g_string_new(next_voice->d_name);
				gtk_signal_connect(GTK_OBJECT(radio_button),"clicked",GTK_SIGNAL_FUNC(on_radio_clicked),voice_name);
				gtk_signal_connect(GTK_OBJECT(radio_button),"destroy",GTK_SIGNAL_FUNC(on_radio_destroy),voice_name);
				gtk_box_pack_start(GTK_BOX(voices_vbox),radio_button,FALSE,TRUE,3);
				radio_group= gtk_radio_button_group(GTK_RADIO_BUTTON(radio_button));
				
			}
			closedir(lang_dir);
		}
		closedir(dir);
	}
	
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(stretch_duration_spinbutton),stretch_duration);
	if(!can_activate_voice && radio_group) {
		GtkRadioButton *default_radio;
		default_radio= GTK_RADIO_BUTTON( g_slist_nth(radio_group,0)->data );
		gtk_button_clicked( GTK_BUTTON(default_radio) );
	}

	g_string_free(base_dir_name,TRUE);
	g_string_free(lang_dir_name,TRUE);
	g_assert(window != 0);
	gtk_signal_connect(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(on_window_destroy), NULL);
	gtk_widget_show_all(window);
}
